#include "display.h"
#include "main.h"
#include "drawtest.h"
int main(void)
{
  // Clear LED array
  led_clear();
  // setup pins and Timer 1
  ioinit();
  // set current_row to 0 to make sure it's in range 0-7
  current_row=0;
  // start interrupts to enable frame buffer updates
  interrupts();
  // Run some tests on led array
  while(1)
    draw_test();
}
// ioinit routine
// set up 3 pins as outputs and configure Timer 1
void ioinit (void)
{
  pinMode(PIN_DATA,OUTPUT);
  pinMode(PIN_LATCH,OUTPUT);
  pinMode(PIN_STORAGE,OUTPUT);

  // Timer 1
  // Frame buffer interrupt
  // 8000000/256/50 = 600 interrupts per second
  OCR1A = 50; 	// interrupt at counter = 10
  TCCR1B |= (1 << CS12); // Prescaler = 128.
  TCCR1B |= (1 << WGM12); // CTC mode. Reset counter when OCR2 is reached.
  TCNT1 = 0x00;	// initial counter value = 0;
  TIMSK1 |= (1 << OCIE1A); // Enable CTC interrupt
}
// shiftout.  Sends data_out onto 74HC595 one bit at a time starting with LSb
void shiftout(unsigned char data_out)
{
  unsigned char i;
  // Shift data onto pins one bit at a time
  for (i=0;i<8;i++)
  {
    // Set LATCH low
    digitalWrite(PIN_LATCH,LOW);
    // Write low bit out onto data pin
    digitalWrite(PIN_DATA,data_out&0x01);
    // Set LATCH high, shifting data in
    digitalWrite(PIN_LATCH,LOW);
    // Right shift data
    data_out=data_out>>1;
  }
}
// Interrupt routine to drive LED 74XX595 latches
// Each cycle (every 1/600th of a second) it sends a single row worth
// of data along with the row selector byte with the relevant row's bit LOW
//
// With the selector 595 on the cathode, only one row's cathodes will be LOW
// Any high bits on the data 595 will light up that row's LEDs.
ISR(TIMER1_COMPA_vect)
{
  // Set storage pin to LOW
  digitalWrite(PIN_STORAGE,LOW);
  // write out data bits first
  shiftout(leds[current_row]);
  // write out column enable bits.  Only ONE should be low
  shiftout(0xff^(1<<current_row));
  // Set storage pin to HIGH, driving latched data onto output pins
  digitalWrite(PIN_STORAGE,HIGH);
  // Increment row count
  current_row++;
  // We want to count from 0-7, so set it to 0 when it reaches 8.
  // Using &0x07 saves a comparison.  0x08&0x07=0x00.  :o)
  current_row&=0x07;
}
